@@ -890,7 +890,7 @@ const common::Mesh *loadMesh(const sdf::Mesh &_meshSdf)
   if (mesh && _meshSdf.Optimization() != sdf::MeshOptimization::NONE)
   {
     const common::Mesh *optimizedMesh = optimizeMesh(_meshSdf, *mesh);
-    if (optimizedMesh)
+    if (optimizedMesh && optimizedMesh->SubMeshCount() > 0u)
       return optimizedMesh;
     else
       gzwarn << "Failed to optimize Mesh " << mesh->Name() << std::endl;
