@@ -675,7 +675,16 @@ void ComponentInspector::Update(const UpdateInfo &,
       auto comp = _ecm.Component<components::Gravity>(this->dataPtr->entity);
       if (comp)
       {
-        setData(item, comp->Data());
+        if (nullptr == item)
+            return;
+        math::Vector3d _gravity = comp->Data();
+        item->setData(QString("Gravity"),
+            ComponentsModel::RoleNames().key("dataType"));
+        item->setData(QList({
+          QVariant(_gravity.X()),
+          QVariant(_gravity.Y()),
+          QVariant(_gravity.Z())
+        }), ComponentsModel::RoleNames().key("data"));
         setUnit(item, "m/s\u00B2");
       }
     }
@@ -1110,6 +1119,31 @@ void ComponentInspector::OnLight(
   this->dataPtr->node.Request(lightConfigService, req, cb);
 }
 
+/////////////////////////////////////////////////
+void ComponentInspector::OnGravity(double _x, double _y, double _z)
+{
+  std::function<void(const msgs::Boolean &, const bool)> cb =
+      [](const msgs::Boolean &/*_rep*/, const bool _result)
+  {
+    if (!_result)
+        gzerr << "Error setting gravity" << std::endl;
+  };
+
+  msgs::Physics req;
+  req.mutable_gravity()->set_x(_x);
+  req.mutable_gravity()->set_y(_y);
+  req.mutable_gravity()->set_z(_z);
+  auto gravityService = "/world/" + this->dataPtr->worldName
+      + "/set_physics";
+  gravityService = transport::TopicUtils::AsValidTopic(gravityService);
+  if (gravityService.empty())
+  {
+    gzerr << "Invalid gravity command service topic provided" << std::endl;
+    return;
+  }
+  this->dataPtr->node.Request(gravityService, req, cb);
+}
+
 /////////////////////////////////////////////////
 void ComponentInspector::OnPhysics(double _stepSize, double _realTimeFactor)
 {
