@@ -1392,8 +1392,8 @@ void ComponentInspectorEditor::OnLoadMesh(const QString &_entity,
 
     if (!common::MeshManager::Instance()->IsValidFilename(meshStr))
     {
-      QString errTxt = QString::fromStdString("Invalid URI: " + meshStr +
-        "\nOnly mesh file types DAE, OBJ, and STL are supported.");
+      gzerr << "Invalid URI: " << meshStr <<
+        "\nOnly mesh file types DAE, OBJ, and STL are supported.\n";
       return;
     }
 
@@ -1406,6 +1406,10 @@ void ComponentInspectorEditor::OnLoadMesh(const QString &_entity,
         gz::gui::App()->findChild<gz::gui::MainWindow *>(),
         &addEntityEvent);
   }
+  else
+  {
+    gzerr << meshStr << " should be local file\n";
+  }
 }
 
 // Register this plugin
