@@ -171,15 +171,12 @@ void SelectEntitiesPrivate::HandleEntitySelection()
       this->selectedEntitiesID.push_back(this->selectedEntitiesIDNew[i]);
 
       Entity entityId = kNullEntity;
-      try
+      if (std::holds_alternative<uint64_t>(
+        visualToHighLight->UserData("gazebo-entity")))
       {
         entityId = std::get<uint64_t>(
           visualToHighLight->UserData("gazebo-entity"));
       }
-      catch(std::bad_variant_access &_e)
-      {
-        // It's ok to get here
-      }
 
       this->selectedEntities.push_back(entityId);
 
@@ -212,14 +209,10 @@ void SelectEntitiesPrivate::HandleEntitySelection()
   }
 
   Entity entityId = kNullEntity;
-  try
+  if (std::holds_alternative<uint64_t>(visual->UserData("gazebo-entity")))
   {
     entityId = std::get<uint64_t>(visual->UserData("gazebo-entity"));
   }
-  catch(std::bad_variant_access &e)
-  {
-    // It's ok to get here
-  }
 
   this->selectionHelper.selectEntity = entityId;
 
@@ -243,14 +236,10 @@ void SelectEntitiesPrivate::LowlightNode(const rendering::VisualPtr &_visual)
   Entity entityId = kNullEntity;
   if (_visual)
   {
-    try
+    if (std::holds_alternative<uint64_t>(_visual->UserData("gazebo-entity")))
     {
       entityId = std::get<uint64_t>(_visual->UserData("gazebo-entity"));
     }
-    catch(std::bad_variant_access &)
-    {
-      // It's ok to get here
-    }
   }
   if (this->wireBoxes.find(entityId) != this->wireBoxes.end())
   {
@@ -271,14 +260,10 @@ void SelectEntitiesPrivate::HighlightNode(const rendering::VisualPtr &_visual)
   }
 
   Entity entityId = kNullEntity;
-  try
+  if (std::holds_alternative<uint64_t>(_visual->UserData("gazebo-entity")))
   {
     entityId = std::get<uint64_t>(_visual->UserData("gazebo-entity"));
   }
-  catch(std::bad_variant_access &)
-  {
-    // It's ok to get here
-  }
 
   // If the entity is not found in the existing map, create a wire box
   auto wireBoxIt = this->wireBoxes.find(entityId);
@@ -358,14 +343,11 @@ void SelectEntitiesPrivate::SetSelectedEntity(
 
   if (topLevelVisual)
   {
-    try
+    if (std::holds_alternative<uint64_t>(
+      topLevelVisual->UserData("gazebo-entity")))
     {
       entityId = std::get<uint64_t>(topLevelVisual->UserData("gazebo-entity"));
     }
-    catch(std::bad_variant_access &)
-    {
-      // It's ok to get here
-    }
   }
 
   if (entityId == kNullEntity)
@@ -496,6 +478,11 @@ void SelectEntities::LoadConfig(const tinyxml2::XMLElement *)
 /////////////////////////////////////////////////
 bool SelectEntities::eventFilter(QObject *_obj, QEvent *_event)
 {
+  if (!this->dataPtr->scene)
+  {
+    if (_event->type() != gz::gui::events::Render::kType)
+      return QObject::eventFilter(_obj, _event);
+  }
   if (_event->type() == gz::gui::events::LeftClickOnScene::kType)
   {
     gz::gui::events::LeftClickOnScene *_e =
@@ -544,14 +531,11 @@ bool SelectEntities::eventFilter(QObject *_obj, QEvent *_event)
           auto visual = this->dataPtr->scene->VisualByIndex(i);
 
           Entity entityId = kNullEntity;
-          try
+          if (std::holds_alternative<uint64_t>(
+            visual->UserData("gazebo-entity")))
           {
             entityId = std::get<uint64_t>(visual->UserData("gazebo-entity"));
           }
-          catch(std::bad_variant_access &)
-          {
-            // It's ok to get here
-          }
 
           if (entityId == entity)
           {
