@@ -58,10 +58,10 @@
 /// \brief    This class can be used to apply a first order filter on a signal.
 ///           It allows different acceleration and deceleration time constants.
 /// \details
-///           Short reveiw of discrete time implementation of first order system
+///           Short review of discrete time implementation of first order system
 ///           Laplace:
 ///             X(s)/U(s) = 1/(tau*s + 1)
-///           continous time system:
+///           continuous time system:
 ///             dx(t) = (-1/tau)*x(t) + (1/tau)*u(t)
 ///           discretized system (ZoH):
 ///             x(k+1) = exp(samplingTime*(-1/tau))*x(k) + (1 - exp(samplingTime*(-1/tau))) * u(k) // NOLINT
@@ -78,7 +78,7 @@ class FirstOrderFilter {
   {
     T outputState;
     if (_inputState > previousState) {
-      // Calcuate the outputState if accelerating.
+      // Calculate the outputState if accelerating.
       double alphaUp = exp(-_samplingTime / timeConstantUp);
       // x(k+1) = Ad*x(k) + Bd*u(k)
       outputState = alphaUp * previousState + (1 - alphaUp) * _inputState;
@@ -292,7 +292,7 @@ void MulticopterMotorModel::Configure(const Entity &_entity,
   {
     this->dataPtr->actuatorNumber =
       sdfClone->GetElement("motorNumber")->Get<int>();
-    gzwarn << "<motorNumber> is depricated, "
+    gzwarn << "<motorNumber> is deprecated, "
            << "please use <actuator_number>.\n";
   }
   else
@@ -448,14 +448,6 @@ void MulticopterMotorModel::PreUpdate(const UpdateInfo &_info,
     doUpdateForcesAndMoments = false;
   }
 
-  if (!_ecm.Component<components::JointVelocityCmd>(
-      this->dataPtr->jointEntity))
-  {
-    _ecm.CreateComponent(this->dataPtr->jointEntity,
-        components::JointVelocityCmd({0}));
-    doUpdateForcesAndMoments = false;
-  }
-
   if (!_ecm.Component<components::WorldPose>(this->dataPtr->linkEntity))
   {
     _ecm.CreateComponent(this->dataPtr->linkEntity, components::WorldPose());
@@ -683,11 +675,10 @@ void MulticopterMotorModelPrivate::UpdateForcesAndMoments(
       refMotorRotVel = this->rotorVelocityFilter->UpdateFilter(
           this->refMotorInput, this->samplingTime);
 
-      const auto jointVelCmd = _ecm.Component<components::JointVelocityCmd>(
-          this->jointEntity);
-      *jointVelCmd = components::JointVelocityCmd(
-          {this->turningDirection * refMotorRotVel
-                              / this->rotorVelocitySlowdownSim});
+      _ecm.SetComponentData<components::JointVelocityCmd>(
+        this->jointEntity,
+        {this->turningDirection * refMotorRotVel
+                                / this->rotorVelocitySlowdownSim});
     }
   }
 }
