@@ -386,6 +386,9 @@ void OpticalTactilePlugin::PreUpdate(const UpdateInfo &_info,
     // We assume there's only one object being touched
     auto csd = _ecm.Component<components::ContactSensorData>(
       this->dataPtr->sensorCollisionEntity);
+    // Check the validity of contact data to avoid crashes
+    if(!csd || csd->Data().contact_size() == 0)
+      return;
     for (const auto &contact : csd->Data().contact())
     {
       this->dataPtr->objectCollisionEntity =
@@ -530,6 +533,7 @@ void OpticalTactilePluginPrivate::Load(const EntityComponentManager &_ecm)
   int contactSensorCounter = 0;
   sdf::Sensor depthCameraSdf;
   components::Pose depthCameraPose = components::Pose();
+  std::string depthCameraTopic;
   for (const Entity &sensor : sensorsInsideLink)
   {
     if (_ecm.EntityHasComponentType(sensor, components::DepthCamera::typeId))
@@ -538,6 +542,10 @@ void OpticalTactilePluginPrivate::Load(const EntityComponentManager &_ecm)
       depthCameraSdf =
         _ecm.Component<components::DepthCamera>(sensor)->Data();
       depthCameraPose = *(_ecm.Component<components::Pose>(sensor));
+      auto depthCameraTopicComp =
+          _ecm.Component<components::SensorTopic>(sensor);
+      if (depthCameraTopicComp)
+        depthCameraTopic = depthCameraTopicComp->Data();
     }
 
     if (_ecm.EntityHasComponentType(sensor, components::ContactSensor::typeId))
@@ -558,16 +566,6 @@ void OpticalTactilePluginPrivate::Load(const EntityComponentManager &_ecm)
   }
 
   // Store depth camera update rate
-  if (!depthCameraSdf.Element()->HasElement("update_rate"))
-  {
-    if (!this->initErrorPrinted)
-    {
-      gzerr << "Depth camera should have an <update_rate> value "
-        << "(only printed once)" << std::endl;
-      this->initErrorPrinted = true;
-    }
-    return;
-  }
   this->cameraUpdateRate = depthCameraSdf.UpdateRate();
 
   // Depth camera data is float, so convert Pose3d to Pose3f
@@ -581,20 +579,27 @@ void OpticalTactilePluginPrivate::Load(const EntityComponentManager &_ecm)
     depthCameraPose.Data().Rot().Z());
 
   // Configure subscriber for depth camera images
-  if (!depthCameraSdf.Element()->HasElement("topic"))
+  if (depthCameraTopic.empty())
   {
-    gzwarn << "Depth camera publishing to __default__ topic. "
-      << "It's possible that two depth cameras are publishing into the same "
-      << "topic" << std::endl;
-  }
-  else
-  {
-    gzdbg << "Depth camera publishing to "
-      << depthCameraSdf.Topic() << " topic" << std::endl;
+    // get the topic from sdf if the one in sensor topic component is empty
+    depthCameraTopic = depthCameraSdf.Topic();
   }
 
+  if (depthCameraTopic.empty())
+  {
+    if (!this->initErrorPrinted)
+    {
+      gzerr << "Depth camera topic is empty. " << std::endl;
+      this->initErrorPrinted = true;
+    }
+    return;
+  }
+
+  gzdbg << "Depth camera publishing to "
+    << depthCameraTopic << " topic" << std::endl;
+
   std::string topic =
-    "/" + depthCameraSdf.Topic() + "/points";
+    "/" + depthCameraTopic + "/points";
   if (!this->node.Subscribe(topic,
       &OpticalTactilePluginPrivate::DepthCameraCallback, this))
   {
